<?php
/**
 * the index of published articles
 *
 * Most pages of your YACS server are articles.
 * As a result, articles support a rich set of features:
 *
 * [*] The specific abstraction used for this kind of object is described in [script]articles/articles.php[/script].
 *
 * [*] Also, articles can be considered as containers of some sort for images, files, etc.
 * Check [script]articles/article.php[/script], which implements [script]shared/anchor.php[/script].
 *
 * [*] Lastly, if you are looking to extend articles to something else, you should check the overlay interface,
 * implemented in [script]overlays/overlay.php[/script], and then a live example such as
 * [script]overlays/recipe.php[/script] or [script]overlays/poll.php[/script].
 *
 * This index page lists most recent regular articles to any surfer.
 * Here, regular means that these pages have been published, and that they have not hit some deadline.
 *
 * The main menu displays the total number of articles.
 * It also has navigation links to browse past pages.
 * Commands are available to either create a new page, to review submitted articles.
 * Associates can also import some XML data, check the database, or populate saple pages.
 *
 * A list of most popular articles is displayed as a sidebar, if the total number of articles in the database is significant.
 *
 * Optionnally, categories to be displayed at this index page are listed as sidebar boxes.
 *
 * This script secretely features a link to the main RSS feeder for this site, namely:
 *
 * [code]&lt;link rel="alternate" href="http://.../yacs/feeds/rss_2.0.php" title="RSS" type="application/rss+xml" /&gt;[/code]
 *
 * Restrictions apply on this page:
 * - anonymous users can see only active pages (the 'active' field == 'Y')
 * - members can see active and restricted pages ('active field == 'Y' or 'R')
 * - associates can see all published articles
 *
 * Accept following invocations:
 * - index.php (view the 20 most recent articles)
 * - index.php/10 (view articles 200 to 220, in the past)
 * - index.php?page=4 (view articles 80 to 100, back in the past)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';
include_once 'articles.php';
include_once '../feeds/feeds.php'; // some links to newsfeeds

// which page should be displayed
$page = 1;
if(isset($_REQUEST['page']))
	$page = $_REQUEST['page'];
elseif(isset($context['arguments'][0]))
	$page = $context['arguments'][0];
$page = strip_tags($page);

// root cache id for this page
$root_cache_id = 'articles/index.php/'.$page;

// render a previous version, if any
Cache::pass_through($root_cache_id);

// load localized strings
i18n::bind('articles');

// load the skin
load_skin('articles');

// page size
$items_per_page = 50;

// the title of the page
$context['page_title'] = i18n::s('Recent published pages');

// count articles in the database
$stats = Articles::stat();
if($stats['count'])
	$context['page_menu'] = array_merge($context['page_menu'], array('_count' => sprintf(i18n::ns('1&nbsp;page', '%d&nbsp;pages', $stats['count']), $stats['count'])));

// navigation commands for articles, if necessary
if($stats['count'] > $items_per_page) {
	$home = 'articles/index.php';
	if(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'Y'))
		$prefix = $home.'/';
	elseif(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'R'))
		$prefix = $home.'/';
	else
		$prefix = $home.'?page=';
	$context['page_menu'] = array_merge($context['page_menu'], Skin::navigate($home, $prefix, $stats['count'], $items_per_page, $page));
}

// the command to post a new page
if(Surfer::is_associate()
	|| (Surfer::is_member() && (!isset($context['users_without_submission']) || ($context['users_without_submission'] != 'Y'))) ) {
	$context['page_menu'] = array_merge($context['page_menu'], array( 'articles/edit.php' => i18n::s('Create a new page') ));
}

// associates can review submitted articles
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'articles/review.php' => i18n::s('Review queue') ));

// associates may import some XML
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'articles/import.php' => i18n::s('Import articles') ));

// associates may check the database
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'articles/check.php' => i18n::s('Maintenance') ));

// look up the database to find the list of articles
$cache_id = 'articles/index.php#articles_by_date#'.$page;
if(!$text =& Cache::get($cache_id)) {

	// load the layout to use
	switch($context['home_layout']) {
		case 'boxesandarrows':
			include_once 'layout_articles_as_boxesandarrows.php';
			$layout =& new Layout_articles_as_boxesandarrows();
			break;
		default:
			$layout = 'full';
			break;
	}

	// query the database and layout that stuff
	$offset = ($page - 1) * $items_per_page;
	if(!$text = Articles::list_by_date($offset, $items_per_page, $layout))
		$text = '<p>'.i18n::s('Be the first one to create a new page!').'</p>';

	// we have an array to format
	elseif(@count($text))
		$text =& Skin::build_list($text, 'decorated');

	// cache this to speed subsequent queries
	Cache::put($cache_id, $text, 'articles');

}
$context['text'] .= $text;

// side bar with the list of most popular articles, if this server is well populated
if($stats['count'] > $items_per_page) {
	$cache_id = 'articles/index.php#articles_by_hits';
	if(!$text =& Cache::get($cache_id)) {
		if($items = Articles::list_by_hits(0, COMPACT_LIST_SIZE, 'compact'))
			$text =& Skin::build_box(i18n::s('Popular'), Skin::build_list($items, 'compact'), 'extra');
		Cache::put($cache_id, $text, 'articles');
	}
	$context['extra'] .= $text;
}

// side bar with a rss feed, if this server is well populated
if($stats['count'] > $items_per_page) {
	$cache_id = 'articles/index.php#articles_as_rss';
	if(!$text =& Cache::get($cache_id)) {
		$text =& Skin::build_box(i18n::s('Newsfeeds'), sprintf(i18n::s('List articles as a %s, or pull %s.'), Skin::build_link(Feeds::get_url('rss'), i18n::s('RSS feed'), 'xml'), Skin::build_link(Feeds::get_url('articles'), i18n::s('articles full contents'), 'xml')), 'navigation');

		Cache::put($cache_id, $text, 'articles');
	}
	$context['extra'] .= $text;
}

// side boxes for related categories, if any
$cache_id = 'articles/index.php#displayed_categories';
if(!$text =& Cache::get($cache_id)) {

	// one box per category
	include_once '../categories/categories.php';
	if($categories = Categories::list_by_date_for_display('article:index', 0, 7, 'raw')) {
		foreach($categories as $id => $attributes) {

			// link to the category page from the box title
			$label =& Skin::build_box_title(Skin::strip($attributes['title']), Categories::get_url($id), i18n::s('Visit the category'));

			// box content
			if($items = Members::list_articles_by_date_for_anchor('category:'.$id, 0, COMPACT_LIST_SIZE, 'compact'))
				$text .= Skin::build_box($label, Skin::build_list($items, 'compact'), 'navigation')."\n";
		}
	}
	Cache::put($cache_id, $text, 'categories');
}
$context['extra'] .= $text;

// referrals, if any
if(Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y'))) {

	$cache_id = 'articles/index.php#referrals#';
	if(!$text =& Cache::get($cache_id)) {

		// box content
		include_once '../agents/referrals.php';
		$text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].'articles/index.php');

		// in a sidebar box
		if($text)
			$text =& Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

		// save in cache for one hour 60 * 60 = 3600
		Cache::put($cache_id, $text, 'referrals', 3600);

	}

	// in the extra panel
	$context['extra'] .= $text;
}

// a meta link to a feeding page
$context['page_header'] .= "\n".'<link rel="alternate" href="'.$context['url_to_root'].Feeds::get_url('rss').'" title="RSS" type="application/rss+xml"'.EOT;

// render the skin
render_skin($root_cache_id);

?>