<?php
/**
 * layout articles as news
 *
 * This layout is used at the site front page to layout news and featured pages.
 * It is also used at section index pages to layout news.
 *
 * @see index.php
 * @see sections/view.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_articles_as_news extends Layout_interface {

	/**
	 * list articles as news
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('articles');

		// flag articles updated recently
		if($context['site_revisit_after'] < 1)
			$context['site_revisit_after'] = 2;
		$dead_line = gmstrftime('%Y-%m-%d %H:%M:%S', mktime(0,0,0,date("m"),date("d")-$context['site_revisit_after'],date("Y")));
		$now = gmstrftime('%Y-%m-%d %H:%M:%S');

		// we return an array of ($url => $attributes)
		$items = array();

		// process all items in the list
		include_once $context['path_to_root'].'articles/article.php';
		include_once $context['path_to_root'].'comments/comments.php';
		include_once $context['path_to_root'].'files/files.php';
		include_once $context['path_to_root'].'links/links.php';
		include_once $context['path_to_root'].'overlays/overlay.php';
		while($item =& SQL::fetch($result)) {

			// the url to view this item
			$url = Articles::get_url($item['id'], 'view', $item['nick_name']);

			// reset the rendering engine between items
			Codes::initialize($url);

			// get the main anchor
			$anchor = get_anchor($item['anchor']);

			// get the related overlay, if any
			$overlay = Overlay::load($item);

			// initialize variables
			$prefix = $suffix = $icon = '';

			// flag articles that are dead, or created or updated very recently
			if(($item['expiry_date'] > NULL_DATE) && ($item['expiry_date'] <= $now))
				$prefix .= EXPIRED_FLAG;
			elseif($item['create_date'] >= $dead_line)
				$suffix .= NEW_FLAG;
			elseif($item['edit_date'] >= $dead_line)
				$suffix .= UPDATED_FLAG;

			// signal articles to be published
			if(($item['publish_date'] <= NULL_DATE) || ($item['publish_date'] > gmstrftime('%Y-%m-%d %H:%M:%S')))
				$prefix .= DRAFT_FLAG;

			// signal locked articles
			if(isset($item['locked']) && ($item['locked'] == 'Y'))
				$prefix .= LOCKED_FLAG;

			// signal restricted and private articles
			if($item['active'] == 'N')
				$prefix .= PRIVATE_FLAG;
			elseif($item['active'] == 'R')
				$prefix .= RESTRICTED_FLAG;

			// go to a new line
			$suffix .= BR;

			// use the title to label the link
			if(is_object($overlay) && is_callable(array($overlay, 'get_live_title')))
				$label = $overlay->get_live_title($item);
			else
				$label = ucfirst(Codes::beautify(strip_tags($item['title'], '<br><div><img><p><span>')));

			// the introductory text
			if($item['introduction']) {
				$suffix .= Codes::beautify($item['introduction'], $item['options']);

				// add a link to the main page
				$suffix .= BR.Skin::build_link($url, i18n::s('more').MORE_IMG, 'basic', i18n::s('Read the page'));

			// else use a teaser, if no overlay
			} elseif(!is_object($overlay)) {
				$article =& new Article();
				$article->load_by_content($item);
				$suffix .= $article->get_teaser('teaser');
			}

			// insert overlay data, if any
			if(is_object($overlay))
				$suffix .= $overlay->get_text('list', $item);

			// the icon to put in the left column
			if($item['thumbnail_url'])
				$suffix .= BR.Skin::build_link($url, '<img src="'.$item['thumbnail_url'].'" alt="" title="'.encode_field($label).'"'.EOT, 'basic');

			// details
			$details = array();

			// date of last modification -- do not use action code here
			$details[] = Skin::build_date($item['edit_date']);

			// info on related files
			$stats = Files::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;file', '%d&nbsp;files', $stats['count']), $stats['count']);

			// info on related links
			$stats = Links::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;link', '%d&nbsp;links', $stats['count']), $stats['count']);

			// info on related comments
			$stats = Comments::stat_for_anchor('article:'.$item['id']);
			if($stats['count'])
				$details[] = sprintf(i18n::ns('1&nbsp;comment', '%d&nbsp;comments', $stats['count']), $stats['count']);

			// actually insert details
			if($details)
				$suffix .= '<p class="details">'.ucfirst(trim(implode(', ', $details))).'</p>';

			// list all components for this item
			$items[$url] = array($prefix, $label, $suffix, 'article', $icon);

		}

		// end of processing
		SQL::free($result);
		return $items;
	}

}

?>