<?php
/**
 * handle content of the database through the wire
 *
 * @todo allow for the restore of a local file from inbox/restore (like for script staging)
 *
 * This script helps either:
 * - to backup the database, by providing a file made of SQL statements that reflects database content
 * - to restore the database, by processing a set of SQL statements in a file
 * - to change database content, by processing some SQL statements typed directly in a text area
 * - to backup user files, by providing a zip file
 * - to backup user images, by providing a zip file
 * - to backup all YACS parameters files, packed in a single zip file
 * - to backup the current skin, with all files packed in a single zip file
 *
 * With a simple click you can create a backup version of the database contents.
 * Without another click you can restore data when necessary.
 *
 * Returned backup data that are files containing standard SQL-queries,
 * that reconstruct the state of the database as it was when the backup was created.
 *
 * It's strongly encouraged to take a backup regularly (weekly or so).
 *
 * The backup function can be limited to the current table prefix, if a
 * database hosts several instances of YACS.
 *
 * The restoring utility can actually process any set of SQL statements, as long as
 * each statement ends with the character ';' (semicolon).
 * Comment lines starting with the character '#' are also supported.
 *
 * Therefore, you can use the restore facility as a convenient method to inject new records in your database.
 *
 * Another facility has been added to ease bulk changes of the database. You just have to type one or several
 * statements in a web form and hit the submit button.
 * Statements have to be separated with the character ';' (semicolon).
 * Comment lines starting with the character '#' are also supported.
 * This is a simple way to achieve small bulk changes of the database when necessary, without the overhead of creating a file.
 *
 * If you are in trouble with your database, you can try one of following options.
 *
 * Option 1: If you have been authenticated as a valid associate, trigger this script manually
 * from your web browser. Upload the backup file and let YACS process SQL statements.
 *
 * Option 2: When triggered, the script complains that you are not an associate.
 * In this case, use your FTP account to delete the switch file, at the YACS installation directory.
 * The file can have either the name [code]parameters/switch.on[/code] or [code]parameters/switch.off[/code].
 * Then restart the script to make it accept your upload request.
 * On next click you may have a strange, but harmless, label from setup.php because
 * it will automatically recreate [code]parameters/switch.on[/code].
 *
 * Option 3: Use phpMyAdmin, or an equivalent web-based interface to MySQL.
 * Use your web browser to trigger phpMyAdmin, and use the backup file to setup and populate tables.
 *
 * Option 4: use MySQL directly.
 * If you have a shell account, restoring a backup can be done by running the mysql program with the following
 * arguments:
 *
 * [snippet]
 * mysql -u username -p -h hostname databasename < backupfile.sql
 * [/snippet]
 *
 * In all cases, you should run the [script]control/setup.php[/script] after any restore, in order to
 * recreate index files that are used by YACS to fasten access to data.
 *
 * To restore other kinds of backup archives (user files, user images, parameter files, or skin),
 * you have to explode the archive manually and to upload files individually.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @tester Edige
 * @tester Lucrecius
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 *
 * @link http://www.phpMyAdmin.org/
 *
 * @see control/setup.php
 */

// common definitions and initial processing
include_once '../shared/global.php';

// load localized strings
i18n::bind('control');

// load the skin
load_skin('control');

// the path to this page
$context['path_bar'] = array( 'control/' => i18n::s('Control Panel') );

// the title of the page
$context['page_title'] = i18n::s('Backup/restore the database');

// the user table should be empty, or the surfer has to be an associate, or the switch file has been deleted
$query = "SELECT count(*) FROM ".SQL::table_name('users');
if((SQL::query($query) !== FALSE) && !Surfer::is_associate()
	&& (file_exists($context['path_to_root'].'parameters/switch.on') || file_exists($context['path_to_root'].'parameters/switch.off'))) {

	// prevent access to this script
	Safe::header('Status: 403 Forbidden', TRUE, 403);
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

	// forward to the control panel
	$menu = array('control/' => i18n::s('Back to the control panel'));
	$context['text'] .= Skin::build_list($menu, 'menu_bar')."\n";

// backup the database
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'backup')) {

	// file header
	$sql = '#'."\n"
		.'# This file has been created by the configuration script control/backup.php'."\n"
		.'# on '.gmdate("F j, Y, g:i a").' GMT, for '.Surfer::get_name().'.'."\n"
		.'# Please avoid to modify it manually.'."\n"
		.(isset($_REQUEST['backup_prefix']) ? '#'."\n".'# Only tables with the prefix "'.$_REQUEST['backup_prefix'].'" have been considered'."\n" : '')
		.'#'."\n\n";

	//enumerate tables
	$queries = 0;
	$tables = mysql_list_tables($context['database']);
	while($row = SQL::fetch_row($tables)) {

		// table name
		$table_name=$row[0];

		// skip unmatched prefixes
		if(isset($_REQUEST['backup_prefix']) && !preg_match('/'.preg_quote($_REQUEST['backup_prefix'], '/').'/i', $table_name))
			continue;

		// read all lines
		$query = "SELECT * FROM $table_name";
		if(!$result =& SQL::query($query))
			$context['text'] .= Skin::error_pop().BR."\n";

		// build the table creation query
		$sql .= 'DROP TABLE IF EXISTS '.$table_name.";\n\n"
			.'CREATE TABLE '.$table_name." (\n";

		//parse the field info first
		$number_of_fields=mysql_num_fields($result);
		$number_of_rows = SQL::count($result);
		$field_list="";
		for($index=0; $index < $number_of_fields; $index++) {

			$field_name = mysql_field_name($result, $index);
			$field_type = mysql_field_type($result, $index);
			$field_size = mysql_field_len($result, $index);
			$field_flags = mysql_field_flags($result, $index);

			$sql.=" $field_name ";

			$is_numeric = FALSE;
			switch(strtolower($field_type)) {
			case "int":
				$sql .= "int";
				$is_numeric = TRUE;
				break;

			case "blob":
				$sql .= "text";
				$is_numeric = FALSE;
				break;

			case "real":
				$sql .= "real";
				$is_numeric = TRUE;
				break;

			case "string":
				$sql .= "char($field_size)";
				$is_numeric = FALSE;
				break;

			case "unknown":
				switch(intval($field_size)) {
				case 4: //little weakness here...there is no way (thru the PHP/MySQL interface) to tell the difference between a tinyint and a year field type
					$sql .= "tinyint";
					$is_numeric = TRUE;
					break;

				default:	//we could get a little more optimzation here! (i.e. check for medium ints, etc.)
					$sql .= "int";
					$is_numeric = TRUE;
					break;
				}
				break;

			case "timestamp":
				$sql .= "timestamp";
				$is_numeric = TRUE;
				break;

			case "date":
				$sql .= "date";
				$is_numeric = FALSE;
				break;

			case "datetime":
				$sql .= "datetime";
				$is_numeric = FALSE;
				break;

			case "time":
				$sql .= "time";
				$is_numeric = FALSE;
				break;

			default: //future support for field types that are not recognized (hopefully this will work without need for future modification)
				$sql .= $field_type;
				$is_numeric = TRUE; //I'm assuming new field types will follow SQL numeric syntax..this is where this support will breakdown
				break;
			}

			//VERY, VERY IMPORTANT!!! Don't forget to append the flags onto the end of the field creator

			$is_unsigned = FALSE;
			if(strpos($field_flags, "unsigned") != FALSE) {
				//timestamps are a little screwy so we test for them
				if($field_type != "timestamp") {
					$sql .= " unsigned";
					$is_unsigned = TRUE;
				}
			}

			if(strpos($field_flags, "zerofill") != FALSE) {
				//timestamps are a little screwy so we test for them
				if($field_type != "timestamp")
					$sql .= " zerofill";
			}

			if(strpos($field_flags, "auto_increment") != FALSE)
				$sql .= " auto_increment";

			if(strpos($field_flags, "not_null") != FALSE)
				$sql .= " not null";

			if(strpos($field_flags, "primary_key") != FALSE)
				$sql .= " primary key";

			//End of field flags

			if($index < $number_of_fields-1) {
				$sql .= ",\n";
				$field_list .= $field_name.", ";
			} else {
				$sql .= "\n) TYPE=MyISAM;\n\n";
				$field_list .= $field_name;
			}

			//we need some of the info generated in this loop later in the algorythm...save what we need to arrays
			$ina[$index] = $is_numeric;
			$inu[$index] = $is_unsigned;

		}

		//parse out the table's data and generate the SQL INSERT statements in order to replicate the data itself...
		for($c=0; $c < $number_of_rows; $c++) {

			$sql .= "INSERT INTO $table_name ($field_list) VALUES (";

			$row = SQL::fetch_row($result);
			for($d=0; $d < $number_of_fields; $d++) {

				if($inu[$d] == TRUE)
					$sql .= $row[$d]; // unsigned, do not use intval()
				elseif($ina[$d] == TRUE)
					$sql .= intval($row[$d]);
				else
					$sql .= '"'.SQL::escape(strval($row[$d])).'"';

				if($d < ($number_of_fields-1))
					$sql.=", ";

			}

			$sql .= ");\n";

			// ensure we have enough time
			$queries++;
			if(!($queries%100))
				Safe::set_time_limit(30);

		}

		$sql .= "\n\n";

	}
	SQL::free($result);

	// if possible, make a gzip file
	if(is_callable('gzencode')) {

		// suggest a download
		Safe::header('Content-Disposition: attachment; filename="'.$context['host_name'].'_'.$context['database'].'.sql.gz"');

		// compress sql statements using gzip
		$content = gzencode($sql, 9);

		// send gzip file
		Safe::header('Content-Type: application/x-gzip');
		Safe::header('Content-Length: '.strlen($content));
		Safe::header('Content-Transfer-Encoding: binary'); // already encoded
		echo $content;

	// else send an ascii file
	} else {

		// suggest a download
		Safe::header('Content-Disposition: attachment; filename="'.$context['host_name'].'_'.$context['database'].'.sql"');

		// send sql statements as-is
		Safe::header('Content-Type: application/octet-stream');
		Safe::header('Content-Length: '.strlen($sql));
		echo $sql;

	}

	// remember this in log as well
	Logger::remember('control/backup.php', 'The database has been saved');

	// do not allow for regular rendering
	exit;

// restore the database
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'restore')) {

	// restoring the database
	$context['text'] .= '<p>'.i18n::s('Restoring the database...')."</p>\n";

	// no file has been uploaded
	if(!$_FILES['upload']['name'] || ($_FILES['upload']['name'] == 'none'))
		$context['text'] .= '<p>'.i18n::s('No file has been uploaded')."</p>\n";

	// no bytes have been transmitted
	elseif(!$_FILES['upload']['size']) {
		$context['text'] .= '<p>'.i18n::s('Zero bytes transmitted')."</p>\n";

		// access the temporary uploaded file
		$file_upload = $_FILES['upload']['tmp_name'];

	// check provided upload name
	} elseif(!Safe::is_uploaded_file($_FILES['upload']['tmp_name']))
		$context['text'] .= '<p>'.sprintf(i18n::s('Possible file attack on %s'), $_FILES['upload']['tmp_name'])."</p>\n";

	// uncompress while reading
	elseif(!$handle = gzopen($_FILES['upload']['tmp_name'], 'rb'))
		$context['text'] .= '<p>'.sprintf(i18n::s('Unable to read %s'), $_FILES['upload']['tmp_name'])."</p>\n";

	// process statements
	else {

		// ensure enough execution time
		Safe::set_time_limit(30);

		// process every line
		$query = '';
		$queries = 0;
		$count = 0;
		while(!gzeof($handle)) {

			// lines can be quite big...
			$line = gzgets($handle, 262144);

			// skip line comments
			if(preg_match('/^\s*#/', $line))
				continue;

			// current line, for possible error reporting
			$count++;
			if(!$query)
				$here = $count;

			// a statement
			$query .= trim($line);

			// end of statement - process it
			if(preg_match('/(^DROP|\'\s*;\s*$|\d\s*;\s*$|\)\s*;\s*$|\)\s*type.+;\s*$)/is', $query) || gzeof($handle)) {

				// chop end of statement
				if($query = rtrim($query, "; \t\r\n\0\x0B")) {

					// execute the statement
					if(!SQL::query($query, TRUE) && SQL::errno())
						$context['text'] .= '<p>'.$here.': '.$query.BR.SQL::error()."</p>\n";
					$query = '';
					$queries++;

					// ensure we have enough time
					if(!($queries%50))
						Safe::set_time_limit(30);

				}

			}
		}

		// clear the cache
		Cache::clear();

		// report of script data
		$time = round(get_micro_time() - $context['start_time'], 2);
		$context['text'] .= '<p>'.sprintf(i18n::s('%d SQL statements have been processed in %.2f seconds.'), $queries, $time).'</p>';

		// also delete files if requested to do so
		if(isset($_REQUEST['delete_files']) && ($_REQUEST['delete_files'] == 'Y')) {
			$milestone = get_micro_time();

			/**
			 * delete a directory and all of its content
			 *
			 * @param string the directory to delete
			 */
			function delete_all($path) {
				global $context;

				$path_translated = str_replace('//', '/', $context['path_to_root'].'/'.$path);
				if($handle = Safe::opendir($path_translated)) {

					while(($node = Safe::readdir($handle)) !== FALSE) {

						if($node == '.' || $node == '..')
							continue;

						// make a real name
						$target = str_replace('//', '/', $path.'/'.$node);
						$target_translated = str_replace('//', '/', $path_translated.'/'.$node);

						// delete a sub directory
						if(is_dir($target_translated))
							delete_all($path.'/'.$node);

						// delete the node
						Safe::unlink($context['path_to_root'].$path.'/'.$node);
						Safe::rmdir($context['path_to_root'].$path.'/'.$node);
						global $deleted_nodes;
						$deleted_nodes++;

					}
					Safe::closedir($handle);
				}

			}

			// locations with user files
			$paths = array();
			$paths[] = 'files/article';
			$paths[] = 'files/category';
			$paths[] = 'files/section';
			$paths[] = 'files/user';
			$paths[] = 'images/article';
			$paths[] = 'images/category';
			$paths[] = 'images/section';
			$paths[] = 'images/user';

			$context['text'] .= '<p>'.i18n::s('Purge of files and images...').'</p><ul>';
			foreach($paths as $path) {
				$context['text'] .= '<li>'.$path.'</li>';
				delete_all($path);
			}
			$context['text'] .= '</ul>';

			// report of script data
			global $deleted_nodes;
			if($deleted_nodes) {
				$time = round(get_micro_time() - $milestone, 2);
				$context['text'] .= '<p>'.sprintf(i18n::s('%d files and images have been removed from the file system in %.2f seconds.'), $deleted_nodes, $time).'</p>';
			}

		}

		// recreates indes as well
		$menu = array('control/setup.php?action=build' => i18n::s('Reindex tables and optimize data storage'));
		$context['text'] .= Skin::build_list($menu, 'menu_bar')."\n";

		// remember this in log as well
		Logger::remember('control/backup.php', 'The database has been restored', $queries.' SQL statements have been processed in '.$time.' seconds.');

	}

// update the database
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'update')) {

	// updating the database
	$context['text'] .= '<p>'.i18n::s('Updating the database...')."</p>\n";

	// no file has been uploaded
	if(!$_REQUEST['statements'])
		$context['text'] .= '<p>'.i18n::s('No statement has been uploaded.')."</p>\n";

	// else read plain content
	elseif(!$lines = explode("\n", $_REQUEST['statements']))
		$context['text'] .= '<p>'.i18n::s('No statement has been uploaded.')."</p>\n";

	// process statements
	else {

		// ensure enough execution time
		Safe::set_time_limit(30);

		// process every line
		$query = '';
		$queries = 0;
		$count = 0;
		foreach($lines as $line) {

			// skip line comments
			if(preg_match('/^\s*#/', $line))
				continue;

			// current line, for possible error reporting
			$count++;
			if(!$query)
				$here = $count;

			// a statement
			$query .= trim($line);

			// end of statement - process it
			if(preg_match('/(^DROP|\'\s*;\s*$|\d\s*;\s*$|\)\s*;\s*$|\)\s*type.+;\s*$)/is', $query) || ($count >= count($lines))) {

				// chop end of statement
				$query = rtrim($query, "; \t\r\n\0\x0B");

				$context['text'] .= '<pre>'.$query.'</pre>'."\n";

				// execute the statement
				if(($affected = SQL::query($query, TRUE)) === FALSE)
					$context['text'] .= '<p>'.$here.': '.$query.BR.SQL::error()."</p>\n";

				// display query and results
				else
					$context['text'] .= '<p>'.sprintf(i18n::s('%d records have been processed'), $affected)."</p>\n";

				// next query
				$query = '';
				$queries++;

				// ensure we have enough time
				if(!($queries%1000))
					Safe::set_time_limit(30);

			}
		}

		// clear the cache
		Cache::clear();

		// report of script data
		$time_end = get_micro_time();
		$time = round($time_end - $context['start_time'], 2);
		$context['text'] .= '<p>'.sprintf(i18n::s('%d SQL statements have been processed in %.2f seconds.'), $queries, $time).'</p>';

		// recreates indes as well
		$menu = array('control/setup.php' => i18n::s('Reindex tables and optimize data storage'));
		$context['text'] .= Skin::build_list($menu, 'menu_bar')."\n";

		// remember this in log as well
		Logger::remember('control/backup.php', 'The database has been updated', $queries.' SQL statements have been processed in '.$time.' seconds.');

	}

// backup user files
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'files')) {

	// list files
	$context['text'] .= '<p>'.i18n::s('Listing all files...').BR."\n";

	// locate files
	include_once '../scripts/scripts.php';
	$file_path = 'files/';
	$datafiles = Scripts::list_files_at($file_path);

	if(is_array($datafiles)) {
		$context['text'] .= BR.sprintf(i18n::s('%d files have been found.'), count($datafiles)).'</p>'."\n";

		// splash message
		$context['text'] .= '<p>'.i18n::s('On-going archive preparation...').'</p>'."\n";

		// build a zip archive
		include_once '../shared/zipfile.php';
		$zipfile = new zipfile();

		// process every files/ file
		$index = 0;
		foreach($datafiles as $datafile) {

			// let's go
			list($path, $filename) = $datafile;
			$path = str_replace(array('files/', 'file'), '', $path);
			if($path)
				$file = $path.'/'.$filename;
			else
				$file = $filename;

			// do not include icons path
			if($path == 'icons')
				continue;

			// do not include php scripts
			if(preg_match('/\.(php|php\.bak)$/i', $file))
				continue;

			// read file content
			if(($content = Safe::file_get_contents('../'.$file_path.$file)) !== FALSE) {

				// store binary data
				$zipfile->store('files/'.$file, Safe::filemtime('../'.$file_path.$file), $content);

				// avoid timeouts
				if(!(($index++)%50)) {
					Safe::set_time_limit(30);
					SQL::ping();
				}
			}
		}

		// suggest a download
		Safe::header('Content-Type: application/zip');
		Safe::header('Content-Disposition: attachment; filename="backup_files.zip"');

		// send the archive content
		echo $zipfile->get();

		// do not allow for regular rendering
		return;

	// no file
	} else
		$context['text'] .= BR.i18n::s('No file have been found.').'</p>'."\n";

// backup user images
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'images')) {

	// list images
	$context['text'] .= '<p>'.i18n::s('Listing all images...').BR."\n";

	// locate images
	include_once '../scripts/scripts.php';
	$file_path = 'images/';
	$datafiles = Scripts::list_files_at($file_path);

	if(is_array($datafiles)) {
		$context['text'] .= BR.sprintf(i18n::s('%d images have been found.'), count($datafiles)).'</p>'."\n";

		// splash message
		$context['text'] .= '<p>'.i18n::s('On-going archive preparation...').'</p>'."\n";

		// build a zip archive
		include_once '../shared/zipfile.php';
		$zipfile = new zipfile();

		// process every images/ file
		$index = 0;
		foreach($datafiles as $datafile) {

			// let's go
			list($path, $filename) = $datafile;
			$path = str_replace(array('images/', 'images'), '', $path);
			if($path)
				$file = $path.'/'.$filename;
			else
				$file = $filename;

			// do not include php scripts
			if(preg_match('/\.(php|php\.bak)$/i', $file))
				continue;

			// read file content
			if(($content = Safe::file_get_contents('../'.$file_path.$file)) !== FALSE) {

				// store binary data
				$zipfile->store('images/'.$file, Safe::filemtime('../'.$file_path.$file), $content);

				// avoid timeouts
				if(!(($index++)%50)) {
					Safe::set_time_limit(30);
					SQL::ping();
				}
			}
		}

		// suggest a download
		Safe::header('Content-Type: application/zip');
		Safe::header('Content-Disposition: attachment; filename="backup_images.zip"');

		// send the archive content
		echo $zipfile->get();

		// do not allow for regular rendering
		return;

	// no file
	} else
		$context['text'] .= BR.i18n::s('No image have been found.').'</p>'."\n";

// backup parameter files
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'parameters')) {

	// splash message
	$context['text'] .= '<p>'.i18n::s('On-going archive preparation...').'</p>'."\n";

	// build a zip archive
	include_once '../shared/zipfile.php';
	$zipfile = new zipfile();

	// list parameter files
	if(($files = Safe::glob($context['path_to_root'].'parameters/*.include.php')) !== FALSE) {

		// process every parameter file
		foreach($files as $file) {

			// store binary data
			if(($content = Safe::file_get_contents($file)) !== FALSE)
				$zipfile->store(basename($file), Safe::filemtime($file), $content);

		}
	}

	// suggest a download
	Safe::header('Content-Type: application/zip');
	Safe::header('Content-Disposition: attachment; filename="backup_parameters.zip"');

	// send the archive content
	echo $zipfile->get();

	// do not allow for regular rendering
	return;

// backup current skin
} elseif(isset($_REQUEST['action']) && ($_REQUEST['action'] == 'skin')) {

	// list files
	$context['text'] .= '<p>'.i18n::s('Listing all files of the current skin...').BR."\n";

	// locate skin
	include_once '../scripts/scripts.php';
	$file_path = $context['skin'];
	$datafiles = Scripts::list_files_at($file_path.'/');

	if(is_array($datafiles)) {
		$context['text'] .= BR.sprintf(i18n::s('%d files have been found.'), count($datafiles)).'</p>'."\n";

		// splash message
		$context['text'] .= '<p>'.i18n::s('On-going archive preparation...').'</p>'."\n";

		// build a zip archive
		include_once '../shared/zipfile.php';
		$zipfile = new zipfile();

		// process every skin/current_skin/ file
		$index = 0;
		foreach($datafiles as $datafile) {

			// let's go
			list($path, $filename) = $datafile;
			$path = str_replace(array($file_path.'/', $file_path), '', $path);
			if($path)
				$file = $path.'/'.$filename;
			else
				$file = $filename;

			// read file content
			if(($content = Safe::file_get_contents('../'.$file_path.'/'.$file)) !== FALSE) {

				// store binary data
				$zipfile->store($file, Safe::filemtime('../'.$file_path.'/'.$file), $content);

				// avoid timeouts
				if(!(($index++)%50)) {
					Safe::set_time_limit(30);
					SQL::ping();
				}
			}
		}

		// suggest a download
		Safe::header('Content-Type: application/zip');
		Safe::header('Content-Disposition: attachment; filename="backup_'.$context['skin'].'.zip"');

		// send the archive content
		echo $zipfile->get();

		// do not allow for regular rendering
		return;

	// no file
	} else
		$context['text'] .= BR.i18n::s('No file has been found in skin.').'</p>'."\n";

// select the operation to perform
} else {

	// the splash label
	$context['text'] .= '<p>'.i18n::s('Please select below the operation to perform.')."</p>\n";

	// backup
	$context['text'] .= Skin::build_block(i18n::s('Backup database content'), 'title');

	// introductory text
	$context['text'] .= '<p>'.i18n::s('This script will generate SQL statements necessary to rebuild your database from scratch. Please file it at your computer for backup purpose.')."</p>\n";

	// backup optimization
	$context['text'] .= '<p>'.sprintf(i18n::s('Since the full content of your database will be downloaded, you would like to %s it before proceeding.'), Skin::build_link('control/purge.php', i18n::s('purge'), 'shortcut'))."</p>\n";

	// use the same script
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'" id="main_form">';

	// table prefix
	if(isset($context['table_prefix']) && $context['table_prefix']) {
		$context['text'] .= '<p><input type="checkbox" name="backup_prefix" value="'.$context['table_prefix'].'" checked="checked"'.EOT.' '.sprintf(i18n::s('Consider only tables with the prefix %s'), $context['table_prefix'])."</p>\n";
	}

	// the submit button
	$context['text'] .= '<p>'
		.'<input type="hidden" name="action" value="backup" />'
		.Skin::build_submit_button(i18n::s('Yes, I want to download database content'), NULL, NULL, 'go')."\n"
		.'</p></form>';

	// set the focus on the backup button
	$context['text'] .= '<script type="text/javascript">'."\n"
		.'<!--'."\n"
		.'document.getElementById(\'go\').focus();'."\n"
		.'// -->'."\n"
		.'</script>'."\n";

	// this may take several minutes
	$context['text'] .= '<p>'.i18n::s('When you will click on the button the server will be immediately requested to proceed. However, because of the so many things to do on the back-end, you may have to wait for minutes before getting a response displayed. Thank you for your patience.')."</p>\n";

	// restore
	$context['text'] .= Skin::build_block(i18n::s('Restore database content'), 'title');

	// introductory text
	$context['text'] .= '<p>'.i18n::s('Use this script to upload and process a set of SQL statements. WARNING!!! If you upload a backup file existing data will be destroyed prior the restauration.')."</p>\n";

	// the form to restore a file
	$context['text'] .= '<form method="post" enctype="multipart/form-data" action="'.$context['script_url'].'">'
		.'<input type="hidden" name="action" value="restore" />';

	// the maximum size for uploads
	$file_maximum_size = str_replace('M', '000000', Safe::get_cfg_var('upload_max_filesize'));
	if(!$file_maximum_size || ($file_maximum_size > 50000000))
		$file_maximum_size = 5000000;

	// select a file
	$size_hint = preg_replace('/000$/', 'k', preg_replace('/000000$/', 'M', $file_maximum_size));
	$context['text'] .= '<p>'.i18n::s('Select the file to upload')
		.' (&lt;&nbsp;'.$size_hint.'&nbsp;'.i18n::s('bytes').')'.BR
		.' <input type="hidden" name="MAX_FILE_SIZE" value="'.$file_maximum_size.'">'
		.'<input type="file" name="upload" size="30" /></p>';

	// purge files and images
	$context['text'] .= '<p><input type="checkbox" name="delete_files" value="Y"'.EOT.' '.i18n::s('Delete files and images related to previous database content').'</p>';

	// the submit button
	$context['text'] .= '<p>'.Skin::build_submit_button(i18n::s('Apply these SQL statements'), i18n::s('Press [s] to submit data'), 's').'</p>'."\n";

	// end of the form
	$context['text'] .= '</form>';

	// restore tip
	$context['text'] .= '<p>'.i18n::s('You can also upload any text file containing SQL statements to achieve bulk changes of your database.')."</p>\n";

	// this may take several minutes
	$context['text'] .= i18n::s('When you will click on the button the server will be immediately requested to proceed. However, because of the so many things to do on the back-end, you may have to wait for minutes before getting a response displayed. Thank you for your patience.');

	// update
	$context['text'] .= Skin::build_block(i18n::s('Update database content'), 'title');

	// introductory text
	$context['text'] .= '<p>'.i18n::s('Type one or several SQL statements below to change the content of the database. WARNING!!! Be sure to understand the conceptual data model before proceeding, else you would corrupt database content.')."</p>\n";

	// the form to apply statements
	$context['text'] .= '<form method="post" enctype="multipart/form-data" action="'.$context['script_url'].'">'
		.'<input type="hidden" name="action" value="update" />';

	// direct statements
	$context['text'] .= i18n::s('UPDATE commands separated by ;').BR
		.'<textarea name="statements" rows="12" cols="50"></textarea>';

	// the submit button
	$context['text'] .= '<p>'.Skin::build_submit_button(i18n::s('Apply these SQL statements'), i18n::s('Press [s] to submit data'), 's').'</p>'."\n";

	// end of the form
	$context['text'] .= '</form>';

	// this may take several minutes
	$context['text'] .= '<p>'.i18n::s('When you will click on the button the server will be immediately requested to proceed. However, because of the so many things to do on the back-end, you may have to wait for minutes before getting a response displayed. Thank you for your patience.')."</p>\n";

	// backup data
	$context['text'] .= Skin::build_block(i18n::s('Backup data'), 'title');

	// introductory text
	$context['text'] .= '<p>'.i18n::s('Use this script to download archives of all files, images, parameters or current skin. Please file them at your computer for backup purpose.').'</p>'."\n";

	// the submit button for files
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'">';
	$context['text'] .= '<p>'
		.'<input type="hidden" name="action" value="files" />'
		.Skin::build_submit_button(i18n::s('Yes, I want to download files'), i18n::s('Press [f] to save files'), 'f')
		.'</p>'."\n".'</form>';

	// the submit button for images
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'">';
	$context['text'] .= '<p>'
		.'<input type="hidden" name="action" value="images" />'
		.Skin::build_submit_button(i18n::s('Yes, I want to download images'), i18n::s('Press [i] to save images'), 'f')
		.'</p>'."\n".'</form>';

	// the submit button for parameters
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'">';
	$context['text'] .= '<p>'
		.'<input type="hidden" name="action" value="parameters" />'
		.Skin::build_submit_button(i18n::s('Yes, I want to download parameter files'), i18n::s('Press [p] to save parameters'), 'f')
		.'</p>'."\n".'</form>';

	// the submit button for current skin
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'">';
	$context['text'] .= '<p>'
		.'<input type="hidden" name="action" value="skin" />'
		.Skin::build_submit_button(i18n::s('Yes, I want to download the current skin'), i18n::s('Press [k] to save the skin'), 'k')
		.'</p>'."\n".'</form>';

	// this may take several minutes
	$context['text'] .= '<p>'.i18n::s('When you will click on the button the server will be immediately requested to proceed. However, because of the so many things to do on the back-end, you may have to wait for minutes before getting a response displayed. Thank you for your patience.').'</p>'."\n";

}

// render the skin
render_skin();

?>