<?php
/**
 * speed up things by caching information
 *
 * @todo add ability to rely on memory cache http://www.rooftopsolutions.nl/article/107
 *
 * The objective of this cache module is to save time.
 * Any other module may use it freely by calling member functions as described in the following example.
 *
 * From within the page that has to cache information:
 * [php]
 * // the id for the cache combines the script name, plus unique information from within this script
 * $cache_id = 'my_module/index.php#items_by_date';
 *
 * // retrieve the information from cache if possible
 * if(!$text =& Cache::get($cache_id)) {
 *
 *   // else build the page dynamically
 *   $result = Items::list_by_date();
 *   $text = Skin::build_list($result, 'decorated');
 *
 *   // cache information for next request
 *   Cache::put($cache_id, $text, 'items');
 * }
 * [/php]
 *
 * The cache will be killed in case of updates:
 * [php]
 * // post a new item
 * function post($fields) {
 *   // the UPDATE statement here
 *   ...
 *
 *   // kill cached items
 *   Cache::clear('items');
 * }
 * [/php]
 *
 * Note that the cache also takes into account a maximum expiration delay.
 * By default this delay is of 5 hours, but this can be changed while pushing
 * an object into the cache.
 *
 * Calls to the SQL library are protected because of migration from old versions
 * of the software.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

Class Cache {

	/**
	 * suppress some content from the cache
	 *
	 * This script always clear global or expired objects
	 *
	 * @param mixed the topic(s) to be deleted; if NULL, clear all cache entries
	 */
	function clear($topic=NULL) {
		global $context;

		// the sql back-end may be not available during software updates
		if(!is_callable(array('SQL', 'query')))
			return;

		// clear the whole cache
		$where = '';

		// if several topics have been given, delete all of them
		if(is_array($topic)) {
			foreach($topic as $item) {
				if($where)
					$where .= ' OR ';
				$where .= "(topic LIKE '".SQL::escape($item)."')";
			}

		// if a topic has been provided, delete related records
		} elseif(is_string($topic) && $topic)
			$where = "(topic LIKE '".SQL::escape($topic)."')";

		// clear global objects, expired items, and specified items --do not report on error, if any
		if($where)
			$query = "DELETE FROM ".SQL::table_name('cache')." WHERE (topic LIKE 'global') OR (expiry_date < '".gmstrftime('%Y-%m-%d %H:%M:%S')."') OR ".$where;

		// clear the entire cache
		else
			$query = "DELETE FROM ".SQL::table_name('cache');

		// do the job
		SQL::query($query, TRUE);

	}

	/**
	 * retrieve cached information
	 *
	 * @param string the id of the text to be retrieved
	 * @return string cached information, or NULL if the no accurate information is available for this id
	 */
	function &get($id) {
		global $context;

		// return by reference
		$output = NULL;

		// always disable cache when server is not switched on
		if(!file_exists($context['path_to_root'].'parameters/switch.on'))
			return $output;

		// the sql back-end may be not available during software updates
		if(!is_callable(array('SQL', 'query')))
			return $output;

		// maybe we don't have to cache
		if(isset($context['without_cache']) && ($context['without_cache'] == 'Y'))
			return $output;

		// sanity check
		if(!$id)
			return $output;

		// cached content depends on surfer capability
		$id .= '/'.Surfer::get_capability();

		// cached content depends on selected language
		$id .= '/'.$context['language'];

		// cached content depends on time offset
		$id .= '/'.Surfer::get_gmt_offset();

		// select among available items -- exact match
		$query = "SELECT * FROM ".SQL::table_name('cache')." AS cache"
			." WHERE (cache.id LIKE '".SQL::escape($id)."')";

		// do not report on error
		if(!$item =& SQL::query_first($query, TRUE))
			return $output;

		// check item validity
		if($item['expiry_date'] < gmstrftime('%Y-%m-%d %H:%M:%S'))
			return $output;

		// we have a valid cached item
		$output = $item['text'];
		return $output;
	}

	/**
	 * build a temporary file name
	 *
	 * This function helps to turn the directory of temporary files to a flat space.
	 *
	 * @param string target file path and name
	 * @return string a suitable name for the temporary directory, or NULL
	 */
	function &hash($id) {
		global $context;

		$output = NULL;
		if($id)
			$output = $context['path_to_root'].'temporary/cache_'.str_replace(array('/', '\\'), '_', $id);
		return $output;
	}

	/**
	 * retrieve cached information and pass it through the browser
	 *
	 * This function is aiming to achieve maximum rendering speed, by providing
	 * full page content with only one SQL request.
	 *
	 *
	 * @param string the id of the text to be retrieved
	 *
	 * @see shared/global.php
	 */
	function pass_through($id) {
		global $context;

		// multiple includes -- see scripts/validate.php
		if(headers_sent() || !isset($_SERVER['REQUEST_METHOD']) || ($_SERVER['REQUEST_METHOD'] != 'GET'))
			return;

		// we are probably looking for a taylored page
		if($name = Surfer::get_name())
			$id .= '/'.$name;

		// cache hit
		if($text =& Cache::get($id)) {

			// handle the output correctly
			render_raw('text/html; charset='.$context['charset']);

			// echo full page content
			echo $text;

			// tick, then exit to skip regular processing
			finalize_page(TRUE);

		}

	}

	/**
	 * put something into the cache
	 *
	 * You can store an array or another kind of structured object, but after
	 * serialization.
	 *
	 * The default caching period is 5 hours (actually, 5*60m*60s = 18,000s)
	 *
	 * @param string the id of this item
	 * @param string the content to store
	 * @param string the topic related to this item
	 * @param int the maximum time before expiration, in seconds
	 */
	function put($id, &$text, $topic='global', $duration=18000) {
		global $context;

		// maybe we don't have to cache
		if(isset($context['without_cache']) && ($context['without_cache'] == 'Y'))
			return;

		// the sql back-end may be not available during software updates
		if(!is_callable(array('SQL', 'query')))
			return;

		// cached content depends on surfer capability
		$id .= '/'.Surfer::get_capability();

		// cached content depends on selected language
		$id .= '/'.$context['language'];

		// cached content depends on time offset
		$id .= '/'.Surfer::get_gmt_offset();

		// suppress existing content, if any --do not report on error, if any
		$query = "DELETE FROM ".SQL::table_name('cache')." WHERE id LIKE '".SQL::escape($id)."'";
		SQL::query($query, TRUE);

		// don't cache more than expected
		$expiry = gmstrftime('%Y-%m-%d %H:%M:%S', time()+$duration);

		// cache also empty content
		if(!$text)
			$text = ' ';

		// update the database; do not report on error
		$query = "INSERT INTO ".SQL::table_name('cache')." SET"
			." id='".SQL::escape($id)."',"
			." text='".SQL::escape($text)."',"
			." topic='".SQL::escape($topic)."',"
			." expiry_date='".$expiry."',"
			." edit_date='".gmstrftime('%Y-%m-%d %H:%M:%S')."'";
		SQL::query($query, TRUE);
	}

	/**
	 * create tables for the cache
	 */
	function setup() {
		global $context;

		$fields = array();
		$fields['id']			= "VARCHAR(255) DEFAULT '' NOT NULL";		// up to 255 chars
		$fields['text']			= "MEDIUMTEXT NOT NULL";					// up to 16M chars
		$fields['topic']		= "VARCHAR(64) DEFAULT '' NOT NULL";		// up to 64 chars
		$fields['edit_date']	= "DATETIME";								// modification date
		$fields['expiry_date']	= "DATETIME";								// expiry date

		$indexes = array();
		$indexes['PRIMARY KEY']			= "(id)";
		$indexes['INDEX topic']			= "(topic)";
		$indexes['INDEX edit_date']		= "(edit_date)";
		$indexes['INDEX expiry_date']	= "(expiry_date)";

		return SQL::setup_table('cache', $fields, $indexes);
	}
}

?>