<?php
/**
 * what we know about the current surfer
 *
 * This module uses cookies to control user login, logout, and access
 * to protected resources.
 *
 * When users enter this server and sign on, their related profile is saved
 * into session data. This module is able to change and retrieve these data.
 *
 * The security model is quite simple. The community defined by this system is made of:
 * - associates, that are allowed to do anything on the server
 * - members, that may contribute to the content of the server and may access articles with restricted access.
 * - subscribers, that may read and contribute to the public pages of the server
 * - all other web surfers that may not be identified by the system
 *
 * YACS saves the address of the surfer host, as reported by the network
 * sub-system, in session data.
 *
 * During the login process the root path (e.g., [code]/yacs/[/code]) is saved in session data.
 * This session attribute is checked afterwards to evaluate actual surfer capability.
 * Therefore, even if several YACS instances are installed at the same host, cross-authentication is blocked.
 * For example, if you have two separate installations at [code]/yacs/[/code] and at [code]/yacs_demo/[/code],
 * users authenticating as associates at the demo site would not be considered as being associates at the main instance.
 *
 * On load, and if the surfer is not an associate, this script will strip any
 * HTML tag from the [code]$_REQUEST[/code] array.
 * Note that special HTML characters encoded as Unicode numerical entities are
 * correctly unmasked, to prevent related code injections. (Thanks to Mordread on this).
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author Vincent No&euml;l
 * @author GnapZ
 * @author Christophe Battarel [email]christophe.battarel@altairis.fr[/email]
 * @tester Mordread Wallas
 * @tester Anatoly
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Surfer {

	/**
	 * extend surfer capability
	 *
	 * This function is used to record additional anchors in session space.
	 *
	 * @param string additional anchor editable during this session
	 */
	function add_handle($handle) {

		// sanity check
		if(!$handle)
			return;

		// an array of manageable objects
		if(!isset($_SESSION['surfer_handles']))
			$_SESSION['surfer_handles'] = array();

		// append the new handle if not already present
		if(!in_array($handle, $_SESSION['surfer_handles']))
			$_SESSION['surfer_handles'][] = $handle;

	}

	/**
	 * list sections assigned to this surfer
	 *
	 * If a member is acting as a managing editor for some sections,
	 * this function returns ids of these sections.
	 *
	 * For subscribers, this function will return the list of accessible sections.
	 *
	 * @param int id of the surfer to consider
	 * @param int maximum number of sections to return
	 * @return array ids of managed sections
	 */
	function assigned_sections($id=NULL, $maximum=21) {
		global $context;

		// default to current surfer
		if(!$id)
			$id = Surfer::get_id();

		// sanity check
		if(!$id)
			return array();

		// query the database only once
		static $my_sections;
		if(isset($my_sections))
			return $my_sections;
		$my_sections = array();

		// only consider live sections
		$now = gmstrftime('%Y-%m-%d %H:%M:%S');
		$where = "((sections.expiry_date is NULL)"
			."	OR (sections.expiry_date <= '".NULL_DATE."') OR (sections.expiry_date > '".$now."'))";

		// the list of sections
		$query = "SELECT sections.id FROM ".SQL::table_name('members')." AS members"
			.", ".SQL::table_name('sections')." AS sections"
			." WHERE (members.anchor LIKE 'user:".SQL::escape($id)."')"
			."	AND (members.member_type = 'section')"
			."	AND (members.member_id = sections.id)"
			."	AND (".$where.")"
			." ORDER BY sections.rank, sections.title, sections.edit_date DESC LIMIT 0, ".$maximum;

		// submit a silent query because at setup tables don't exist
		if(($result =& SQL::query($query, TRUE))) {

			// build the list
			while($row =& SQL::fetch($result))
				$my_sections[] = $row['id'];

		}

		// done
		return $my_sections;

	}

	/**
	 * build the navigation menu for this surfer
	 *
	 * This function has to be called from the template, once the skin has been loaded.
	 *
	 * @param string the type of each link
	 * @return a list of $url => array($prefix, $label, $suffix, $type) to be used with Skin::build_list()
	 *
	 * @see skins/skin_skeleton.php
	 */
	function build_user_menu($type = 'submenu') {
		global $context;

		// an array of links
		$menu = array();

		// surfer is a valid user
		if(Surfer::is_logged()) {

			if(Surfer::get_id()) {
				$link = Users::get_url(Surfer::get_id(), 'view', Surfer::get_name());
				$menu[$link] = array(NULL, i18n::s('My profile'), NULL, $type, NULL, i18n::s('View all data this site knows about you'));
			}

			if(Surfer::is_associate()
				|| (Surfer::is_member() && (!isset($context['users_without_submission']) || ($context['users_without_submission'] != 'Y'))) ) {

				$menu['articles/edit.php'] = array(NULL, i18n::s('Create a page'), NULL, $type, NULL, i18n::s('Use a web form to submit new content to this site'));

				// if uploads are administratively allowed
				if(Surfer::may_upload()) {

					$menu['images/edit.php'] = array(NULL, i18n::s('Post an image'), NULL, $type, NULL, i18n::s('Upload a picture in a new page'));

					$menu['files/edit.php'] = array(NULL, i18n::s('Share a file'), NULL, $type, NULL, i18n::s('Upload a file in a new page'));

				}

				$menu['links/edit.php'] = array(NULL, i18n::s('Bookmark a link'), NULL, $type, NULL, i18n::s('Upload a link in a new page'));

			}

			if(Surfer::is_associate())
				$menu['articles/review.php'] = array(NULL, i18n::s('Review queue'), NULL, $type, NULL, i18n::s('Check requests, publish submitted articles, review old pages'));

			$menu['users/logout.php'] = array(NULL, i18n::s('Logout'), NULL, $type, NULL, i18n::s('You will be considered as an anonymous surfer'));

			if(Surfer::is_associate())
				$menu['control/'] = array(NULL, i18n::s('Control Panel'), NULL, $type, NULL, i18n::s('System commands, configuration panels, content overview'));

			$menu['help.php'] = array(NULL, i18n::s('Help'), NULL, $type, NULL, i18n::s('If you don\'t know how to proceed, start here'));

		// surfer has not been authenticated, and self-registration is allowed
		} elseif(!isset($context['users_without_registration']) || ($context['users_without_registration'] != 'Y')) {

			$menu['users/edit.php'] = array(NULL, i18n::s('Register'), NULL, $type, NULL, i18n::s('Create a user profile to be part of this community'));

			$menu['users/login.php'] = array(NULL, i18n::s('Login'), NULL, $type, NULL, i18n::s('Prove who you are'));

			$menu['help.php'] = array(NULL, i18n::s('Help'), NULL, $type, NULL, i18n::s('If you don\'t know how to proceed, start here'));
		}

		if(count($menu))
			return $menu;
		return NULL;
	}

	/**
	 * set default attributes for this surfer
	 *
	 * This function is called before writing records in the database,
	 * to ensure attributes 'edit_name', 'edit_id', 'edit_address' and 'edit_date' have been properly set.
	 *
	 * @param array attributes to check
	 * @return updated attributes
	 *
	 * @see actions/actions.php
	 * @see articles/articles.php
	 * @see categories/categories.php
	 * @see comments/comments.php
	 * @see files/files.php
	 * @see links/links.php
	 * @see locations/locations.php
	 * @see sections/sections.php
	 * @see servers/servers.php
	 * @see tables/tables.php
	 * @see users/users.php
	 */
	function check_default_editor($fields) {

		// default value for editor name
		if(!isset($fields['edit_name']) || !$fields['edit_name'])
			$fields['edit_name'] = Surfer::get_name();

		// default value for editor id
		if(!isset($fields['edit_id']) || !$fields['edit_id'])
			$fields['edit_id'] = Surfer::get_id();

		// default value for editor address
		if(!isset($fields['edit_address']) || !$fields['edit_address'])
			$fields['edit_address'] = Surfer::get_email_address();

		// default value for edition date (GMT)
		if(!isset($fields['edit_date']) || ($fields['edit_date'] <= NULL_DATE))
			$fields['edit_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');

		// done
		return $fields;
	}

	/**
	 * extend capability for this request
	 *
	 * This function is used to flag editors, and any surfer which benefits from
	 * extended rights for the duration of the current script.
	 *
	 * @param string new capability of this surfer, '?', 'M' or 'A'
	 */
	function empower($capability='A') {
		global $context;

		if(($capability == '?') || ($capability == 'M') || ($capability == 'A'))
			$context['empowered'] = $capability;
	}

	/**
	 * adjust a time stamp to surfer time zone
	 *
	 * @param string a time stamp in GMT zone
	 * @return string a time stamp in surfer time zone
	 */
	function &from_GMT($stamp) {
		global $context;

		// sanity check
		if(!isset($stamp) || ($stamp <= NULL_DATE) || (($seconds = strtotime($stamp)) == -1))
			return $stamp;

		// shift to surfer time zone
		$stamp = strftime('%Y-%m-%d %H:%M:%S', $seconds + (Surfer::get_gmt_offset() * 3600));
		return $stamp;
	}

	/**
	 * get the capability of the current surfer
	 *
	 * If the surfer has authenticated, his general capability is stored in
	 * session context.
	 *
	 * Surfer can be empowered on some transactions, and this is saved in
	 * transaction context.
	 *
	 * This function returns highest setting of session and transaction
	 * capability.
	 *
	 * @return char either 'A', 'M', 'S' or '?'
	 */
	function get_capability() {
		global $context;

		// enforce session scope
		if(isset($_SESSION['surfer_capability']) && isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// session data may be overriden by transaction data
			if(($context['empowered'] == 'A') || ($_SESSION['surfer_capability'] == 'A'))
				return 'A';
			if(($context['empowered'] == 'M') || ($_SESSION['surfer_capability'] == 'M'))
				return 'M';
			if(($context['empowered'] == 'S') || ($_SESSION['surfer_capability'] == 'S'))
				return 'S';

		}

		// use transaction data
		return $context['empowered'];
	}

	/**
	 * get the preferred editor
	 *
	 * Provides XHTML snippet to invoke one of the available editors,
	 * based on user preference.
	 *
	 * Curently YACS supports following choices:
	 * - 'yacs' - the default, plain, code-based textarea
	 * - 'fckeditor' - the WYSIWYG, open-source, Internet standard
	 * - 'tinymce' - an alternate WYSIWYG editor
	 *
	 * @link http://www.fckeditor.net/ FCKEditor
	 * @link http://tinymce.moxiecode.com/ TinyMCE
	 *
	 * @param string the name of the editing field
	 * @param string content to be put in the editor
	 * @param boolean TRUE to stick to current editor, FALSE otherwise
	 * @return string to be inserted in the XHTML flow
	 */
	function get_editor($name='description', $value='', $fixed_editor = FALSE) {
		global $context;

		// returned string
		$text = '';

		// fckeditor
		if(isset($_SESSION['surfer_editor']) && ($_SESSION['surfer_editor'] == 'fckeditor') && is_readable($context['path_to_root'].'included/fckeditor/fckeditor.php')) {

			include_once($context['path_to_root'].'included/fckeditor/fckeditor.php');
			$handle =& new FCKeditor($name);
			$handle->BasePath = $context['url_to_root'].'included/fckeditor/';
			$handle->Value = $value;
			$handle->Height = '400' ;
			$handle->ToolbarSet = 'YACS'; // see fckconfig.js
			$text .= $handle->CreateHtml() ;

			// signal an advanced editor
			$text .= '<input type="hidden" name="editor" value="fckeditor" />';

			// editor
			if(!$fixed_editor) {
				$text .= '<select name="preferred_editor" onchange="return document.getElementById(\'main_form\').submit();">'
					.'<option value="yacs">'.i18n::s('Textarea')."</option>\n"
					.'<option value="fckeditor" selected="selected">'.i18n::s('FCKEditor')."</option>\n"
					.'<option value="tinymce">'.i18n::s('TinyMCE')."</option>\n"
					.'</select> ';
			}

			// hint
			$text .= '<span class="tiny">'.sprintf(i18n::s('Powered by %s. You can also select %s or include %s and %s to enhance the rendering of your page.'),
				Skin::build_link('http://www.fckeditor.net/', i18n::s('FCKeditor'), 'external'),
				Skin::build_link('http://tinymce.moxiecode.com/', i18n::s('TinyMCE'), 'external'),
				Skin::build_link('codes/', i18n::s('YACS codes'), 'help'),
				Skin::build_link('smileys/', i18n::s('smileys'), 'help')).'</span>';

		// tinymce
		} elseif(isset($_SESSION['surfer_editor']) && ($_SESSION['surfer_editor'] == 'tinymce') && is_readable($context['path_to_root'].'included/tiny_mce/tiny_mce_src.js')) {

			$text .= '<script language="javascript" type="text/javascript" src="'.$context['url_to_root'].'included/tiny_mce/tiny_mce_src.js"></script>'."\n"
					.'<script language="javascript" type="text/javascript">'."\n"
					.'	tinyMCE.init({'."\n"
					.'		mode : "textareas",'."\n"
					.'		theme : "advanced",'."\n"
					.'		editor_selector : "tinymce",'."\n"
					.'		plugins : "advhr,advimage,advlink,emotions,fullscreen,paste,searchreplace,table",'."\n"
					.'		theme_advanced_buttons1 : "fullscreen,|,cut,copy,paste,pastetext,pasteword,|,search,replace,|,fontselect,fontsizeselect",'."\n"
					.'		theme_advanced_buttons2 : "bold,italic,underline,strikethrough,|,forecolor,backcolor,|,justifyleft,justifycenter,justifyright,justifyfull,|,bullist,numlist,|,outdent,indent,|,undo,redo",'."\n"
					.'		theme_advanced_buttons3 : "tablecontrols,|,link,unlink,anchor,image,advhr",'."\n"
					.'		theme_advanced_toolbar_location : "top",'."\n"
					.'		theme_advanced_toolbar_align : "left",'."\n"
					.'		plugi2n_insertdate_dateFormat : "%Y-%m-%d",'."\n"
					.'		plugi2n_insertdate_timeFormat : "%H:%M:%S",'."\n"
					.'		paste_auto_cleanup_on_paste : true,'."\n"
					.'		paste_convert_headers_to_strong : false,'."\n"
					.'		paste_strip_class_attributes : "all",'."\n"
					.'		paste_remove_spans : false,'."\n"
					.'		paste_remove_styles : false 	'."\n"
					.'	});'."\n"
					.'</script>'."\n";

			// the textarea that will be handled by TinyMCE
			$text .= '<div><textarea name="'.$name.'" id="edit_area" class="tinymce" rows="25" cols="50" accesskey="c">'.encode_field($value).'</textarea></div>';

			// signal an advanced editor
			$text .= '<input type="hidden" name="editor" value="tinymce" />';

			// editor
			if(!$fixed_editor) {
				$text .= '<select name="preferred_editor" onchange="return document.getElementById(\'main_form\').submit();">'
					.'<option value="yacs">'.i18n::s('Textarea')."</option>\n"
					.'<option value="fckeditor">'.i18n::s('FCKEditor')."</option>\n"
					.'<option value="tinymce" selected="selected">'.i18n::s('TinyMCE')."</option>\n"
					.'</select> ';
			}

			// hint
			$text .= '<span class="tiny">'.sprintf(i18n::s('Powered by %s. You can also select %s or include %s and %s to enhance the rendering of your page.'),
				Skin::build_link('http://tinymce.moxiecode.com/', i18n::s('TinyMCE'), 'external'),
				Skin::build_link('http://www.fckeditor.net/', i18n::s('FCKeditor'), 'external'),
				Skin::build_link('codes/', i18n::s('YACS codes'), 'help'),
				Skin::build_link('smileys/', i18n::s('smileys'), 'help')).'</span>';

		// default to plain editor
		} else {
			if(file_exists($context['path_to_root'].'codes/edit.js')) {
				$text .= '<script type="text/javascript" src="'.$context['url_to_root'].'codes/edit.js"></script>';
				if(file_exists($context['path_to_root'].'smileys/edit.js'))
					$text .= '<script type="text/javascript" src="'.$context['url_to_root'].'smileys/edit.js"></script>';
			}
			$text .= '<textarea name="'.$name.'" id="edit_area" rows="25" cols="50" accesskey="c">'.encode_field($value).'</textarea>';

			// editor
			if(!$fixed_editor) {
				$text .= BR.'<select name="preferred_editor" onchange="return document.getElementById(\'main_form\').submit();">'
					.'<option value="yacs" selected="selected">'.i18n::s('Textarea')."</option>\n"
					.'<option value="fckeditor">'.i18n::s('FCKEditor')."</option>\n"
					.'<option value="tinymce">'.i18n::s('TinyMCE')."</option>\n"
					.'</select> ';
			}

			// hint
			$hint = sprintf(i18n::s('Please use %s to enhance the rendering of your page.'), Skin::build_link('codes/', i18n::s('YACS codes'), 'help'));
			if(Surfer::is_associate())
				$hint .= BR.i18n::s('You are allowed to post any XHTML.');
			else
				$hint .= BR.sprintf(i18n::s('Also, following XHTML tags are allowed: %s'), trim(str_replace('><', ', ', $context['users_allowed_tags']), '<>'));
			if(Surfer::is_member()) {
				$hint .= ' '.sprintf(i18n::s('You can switch to %s or %s in %s.'),
					Skin::build_link('http://tinymce.moxiecode.com/', i18n::s('TinyMCE'), 'external'),
					Skin::build_link('http://www.fckeditor.net/', i18n::s('FCKeditor'), 'external'),
					Skin::build_link(Users::get_url(Surfer::get_id(), 'view', Surfer::get_name()), i18n::s('your user profile'), 'basic'));
			}
			$text .= BR.'<span class="tiny">'.$hint.'</span>';
		}

		// job done
		return $text;
	}

	/**
	 * get the e-mail address of the current surfer, if known
	 *
	 * @return string a valid address, or NULL
	 */
	function get_email_address() {
		global $context;

		// use session data
		if(isset($_SESSION['surfer_email_address']))
			return $_SESSION['surfer_email_address'];

		return NULL;
	}

	/**
	 * get offset to GMT
	 *
	 * @return the number of hours (-12 ... +12)
	 *
	 * @see skins/skin_skeleton.php
	 */
	function get_gmt_offset() {
		global $context;

		// enforce session scope
		if(isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// use session data
			if(isset($_SESSION['surfer_gmt_offset']))
				return $_SESSION['surfer_gmt_offset'];

		}
		return 0;
	}

	/**
	 * get the id of the current surfer, if known
	 *
	 * @return a positive integer or 0
	 */
	function get_id() {
		global $context;

		// enforce session scope
		if(isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// use session data
			if(isset($_SESSION['surfer_id']))
				return $_SESSION['surfer_id'];

		}
		return 0;
	}

	/**
	 * get the name of the current surfer, if known
	 *
	 * This function will remove mention of originating server for shadow records.
	 *
	 * @return string user short name, or NULL
	 */
	function get_name() {
		global $context;

		// use session data
		if(isset($_SESSION['surfer_name']))
			return preg_replace('/(@.+)$/', '', $_SESSION['surfer_name']);

		// unknown surfer
		return NULL;
	}

	/**
	 * ask surfer to replicate random data
	 *
	 * This function complements a web form to stop anonymous robots.
	 *
	 * If the global parameter 'users_without_robot_check' is set to 'Y',
	 * or if the surfer has been authenticated,
	 * this function returns NULL.
	 *
	 * In every other cases, the function returns an array to be inserted
	 * in current form
	 *
	 * Example of usage:
	 * [php]
	 * if($field = Surfer::get_robot_stopper())
	 *	   $fields[] = $field;
	 * [/php]
	 *
	 * Data submitted through the form can be checked using function [code]Surfer::may_be_a_robot()[/code].
	 *
	 * @return array to be inserted in the form, or NULL
	 */
	function get_robot_stopper() {
		global $context;

		// we are sure the surfer is not a robot
		if(isset($_SESSION['surfer_is_not_a_robot']) && $_SESSION['surfer_is_not_a_robot'])
			return NULL;

		// we have been asked to not challenge anonymous surfers
		if(isset($context['users_without_robot_check']) && ($context['users_without_robot_check'] == 'Y'))
			return NULL;

		// this only applies to anonymous surfers
		if(Surfer::is_logged())
			return NULL;

		// build a random string --  l, o, O and 0 are confusing
		$pool = 'abcdefghijkmnpqrstuvwxyzABCDEFGHIJKLMNPQRSTUVWXYZ123456789';
		$salt = $pool[mt_rand(0, strlen($pool)-1)]
			.$pool[mt_rand(0, strlen($pool)-1)]
			.$pool[mt_rand(0, strlen($pool)-1)]
			.$pool[mt_rand(0, strlen($pool)-1)]
			.$pool[mt_rand(0, strlen($pool)-1)];

		// add salt and pepper to the form
		$label = i18n::s('Robot stopper').' *';
		$input = i18n::s('Type exactly the following 5 chars:').' '.$salt.' <input type="text" name="pepper" size="7"'.EOT
			.'<input type="hidden" name="salt" value="'.md5($salt).'"'.EOT;
		return array($label, $input);
	}

	/**
	 * is the current user an associate?
	 *
	 * The additional parameter allows for user empowerment on a per-transaction basis.
	 *
	 * @param string specific capability, '?', 'M' or 'A'
	 * @return true or false
	 */
	function is_associate($capability='?') {
		global $context;

		// surfer has been empowered for this transaction
		if($capability == 'A')
			return TRUE;

		// sanity check
		if(!Surfer::is_logged())
			return FALSE;

		// enforce session scope
		if(isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// use session data
			if(isset($_SESSION['surfer_capability']) && ($_SESSION['surfer_capability'] == 'A'))
				return TRUE;

		}
		return FALSE;
	}

	/**
	 * is the surfer a software robot?
	 *
	 * @return TRUE or FALSE
	 */
	function is_crawler() {

		// quite often software robots do not declare themselves
		if(!isset($_SERVER['HTTP_USER_AGENT']))
			return TRUE;

		// well-known robots
		$robots = array(
			'almaden',				// ibm almaden web crawler
			'answerbus',			// http://www.answerbus.com/, web questions
			'ask jeeves',			// ask jeeves
			'baiduspider',			// baiduspider asian search spider
			'blo\.gs',
			'blog', 				// generic ping
			'boitho.com-dc',		// norwegian search engine
			'bot',					// generic bot
			'crawler',				// generic crawler
			'fast-webcrawler',		// all the web
			'frontier',
			'gigabot',				// gigabot
			'googlebot',			// google
			'ia_archiver',			// ia_archiver
			'inktomi',				// inktomi bot
			'mediapartners-google', // google adsense
			'msnbot',				// msn search
			'naverbot',
			'objectssearch',		// open source search engine
			'openbot',				// openbot, from taiwan
			'scooter',				// altavista
			'psbot',				// psbot image crawler
			'slurp',				// inktomi bot
			'sohu-search',			// chinese media company, search component
			'spider',
			'surveybot',
			'teoma',				// ask jeeves
			'yacs',
			'yahoo-verticalcrawler',// old yahoo bot
			'yahoo! slurp', 		// new yahoo bot
			'yahoo-mm', 			// another yahoo bot
			'zyborg'				// looksmart
			);

		// check the user-agent string
		if(preg_match('/('.str_replace('/', '\/', join('|', $robots)).')\b/i', $_SERVER['HTTP_USER_AGENT']))
			return TRUE;

		// maybe a human being, or a dog
		return FALSE;
	}

	/**
	 * is the current surfer the creator of one item?
	 *
	 * To control that a surfer is the actual creator of one article, following code may be used:
	 * [php]
	 * // load the article from the database, including the editors list
	 * $item =& Articles::get($id);
	 *
	 * // check that the current surfer is a valid editor
	 * if(Surfer::is_creator($item['create_id']) {
	 *	 ...
	 * }
	 * [/php]
	 *
	 * @param string the id of the original poster
	 * @return true or false
	 */
	function is_creator($id) {

		// sanity check
		if(!$id)
			return FALSE;
		if(!Surfer::get_id())
			return FALSE;

		// look for strict equivalence
		if(Surfer::get_id() == $id)
			return TRUE;

		return FALSE;
	}

	/**
	 * is this a super user?
	 *
	 * @param string new capability of this surfer, '?', 'M' or 'A'
	 * @return true or false
	 */
	function is_empowered($capability='A') {
		global $context;

		// surfer has been empowered for this transaction
		if($capability == $context['empowered'])
			return TRUE;

		// obvious
		if($context['empowered'] == 'A')
			return TRUE;
		if(Surfer::is_associate())
			return TRUE;
		if(($capability == 'M') && Surfer::is_member())
			return TRUE;

		return FALSE;
	}

	/**
	 * has the current surfer been authenticated?
	 *
	 * @return TRUE or FALSE
	 */
	function is_logged() {
		global $context;

		// enforce session scope
		if(isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// no role has been defined for this surfer
			if(!isset($_SESSION['surfer_capability']))
				return FALSE;

			// surfer has been authenticated as a valid associate
			if($_SESSION['surfer_capability'] == 'A')
				return TRUE;

			// surfer has been authenticated as a valid associate
			if($_SESSION['surfer_capability'] == 'M')
				return TRUE;

			// surfer has been authenticated as a valid associate
			if($_SESSION['surfer_capability'] == 'S')
				return TRUE;

		}
		return FALSE;
	}

	/**
	 * is the current user a member?
	 *
	 * The additional parameter allows for user empowerment on a per-transaction basis.
	 *
	 * @param string specific capability, '?', 'M' or 'A'
	 * @return true or false
	 */
	function is_member($capability='?') {
		global $context;

		// surfer has been empowered for this transaction
		if(($capability == 'A') || ($capability == 'M'))
			return TRUE;

		// sanity check
		if(!Surfer::is_logged())
			return FALSE;

		// enforce session scope
		if(isset($context['url_to_root']) && (!isset($_SESSION['server_id']) || ($_SESSION['server_id'] == $context['url_to_root']))) {

			// use session data
			if(isset($_SESSION['surfer_capability']) && (($_SESSION['surfer_capability'] == 'M') || ($_SESSION['surfer_capability'] == 'A')))
				return TRUE;

		}
		return FALSE;
	}

	/**
	 * check salt and pepper
	 *
	 * This function helps to stop robots, by checking outcome of user challenge.
	 *
	 * It has to be used in cuonjonction with [code]Surfer::get_robot_stopper()[/code].
	 *
	 * @return FALSE if salt and pepper are equals, TRUE otherwise
	 */
	function may_be_a_robot() {
		global $context;

		// this has already been checked
		if(isset($_SESSION['surfer_is_not_a_robot']) && $_SESSION['surfer_is_not_a_robot'])
			return FALSE;

		// we have been asked to not challenge anonymous surfers
		if(isset($context['users_without_robot_check']) && ($context['users_without_robot_check'] == 'Y'))
			return FALSE;

		// surfer has been authenticated
		if(Surfer::is_logged())
			return FALSE;

		// salt could have been hacked
		if(!isset($_REQUEST['salt']))
			return TRUE;

		// salt and pepper are ok
		if(isset($_REQUEST['pepper']) && !strcmp(md5($_REQUEST['pepper']), $_REQUEST['salt'])) {

			// remember this, to not challenge the surfer again
			$_SESSION['surfer_is_not_a_robot'] = TRUE;

			// not a robot, for sure
			return FALSE;
		}

		// user agent is dumb, no doubt on that
		return TRUE;

	}

	/**
	 * back door to some resource
	 *
	 * This function checks the provided handle against authorized handles
	 * listed in session data.
	 *
	 * @param string the secret handle
	 * @return TRUE or FALSE
	 */
	function may_handle($handle) {

		// no handle in session
		if(!isset($_SESSION['surfer_handles']) || !is_array($_SESSION['surfer_handles']))
			return FALSE;

		// true if handle is here
		return in_array($handle, $_SESSION['surfer_handles']);

	}

	/**
	 * check upload capability
	 *
	 * This function checks surfer overall capability against actual
	 * server configuration (PHP parameter 'file_uploads') and also
	 * against the parameter [code]users_without_uploads[/code]
	 * set in the configuration panel for users.
	 *
	 * @see users/configure.php
	 *
	 * @param string actual capability, for possible impersonation (see services/blog.php)
	 * @return TRUE if the surfer is allowed to upload files, FALSE otherwise
	 */
	function may_upload($capability = '?') {
		global $context;

		// server limitation
		if(!ini_get('file_uploads'))
			return FALSE;

		// administrative permission has not been set
		if(!isset($context['users_without_uploads']))
			return TRUE;

		// even associates may not upload files
		if($context['users_without_uploads'] == 'Y')
			return FALSE;

		// only associates may upload files
		if(($context['users_without_uploads'] == 'R') && !Surfer::is_associate() && ($capability != 'A'))
			return FALSE;

		// surfers are encouraged to share files and images
		return TRUE;
	}

	/**
	 * list personal sections for one surfer
	 *
	 * If a member is acting as a managing editor for some sections,
	 * this function returns ids of these sections, if he has created them.
	 *
	 * @param int id of the surfer to consider
	 * @param int maximum number of sections to return
	 * @return array ids of managed sections
	 */
	function personal_sections($id=NULL, $maximum=21) {
		global $context;

		// default to current surfer
		if(!$id)
			$id = Surfer::get_id();

		// sanity check
		if(!$id)
			return array();

		// query the database only once
		static $my_sections;
		if(isset($my_sections))
			return $my_sections;
		$my_sections = array();

		// sections have been created by this surfer
		$where = "(sections.create_id = '".SQL::escape($id)."')";

		// only consider live sections
		$now = gmstrftime('%Y-%m-%d %H:%M:%S');
		$where .= " AND ((sections.expiry_date is NULL)"
			."	OR (sections.expiry_date <= '".NULL_DATE."') OR (sections.expiry_date > '".$now."'))";

		// the list of sections
		$query = "SELECT sections.id FROM ".SQL::table_name('members')." AS members"
			.", ".SQL::table_name('sections')." AS sections"
			." WHERE (members.anchor LIKE 'user:".SQL::escape($id)."')"
			."	AND (members.member_type = 'section')"
			."	AND (members.member_id = sections.id)"
			."	AND ".$where
			." ORDER BY sections.rank, sections.title, sections.edit_date DESC LIMIT 0, ".$maximum;

		// submit a silent query because at setup tables don't exist
		if(($result =& SQL::query($query, TRUE))) {

			// build the list
			while($row =& SQL::fetch($result))
				$my_sections[] = $row['id'];

		}

		// done
		return $my_sections;

	}

	/**
	 * kill the current session
	 *
	 * @link http://fr.php.net/manual/en/function.session-destroy.php PHP: session_destroy
	 */
	function reset() {
		global $context;

		// unset all of the session variables.
		$_SESSION = array();

		// also delete permanent session cookie, if any
		if(isset($_COOKIE['screening'])) {
			Safe::setcookie('screening', '', time()-3600, $context['url_to_root']);

			// also clear cookies used in leading index.php
			if(isset($_ENV['YACS_HOME']))
				Safe::setcookie('screening', '', time()-3600, $_ENV['YACS_HOME'].'/');
			if($context['url_to_root'] == '/yacs/')
				Safe::setcookie('screening', '', time()-3600, '/');
	   }

		// finally, destroy the session and release related resources --no warning if session data cannot be deleted
		if(session_id() && is_callable('session_destroy'))
			@session_destroy();

	}

	/**
	 * surfer has been authenticated
	 *
	 * This function copies user attributes in session storage area.
	 *
	 * Following named attributes from the provided array are copied in session storage area:
	 * - $fields['id'] - id of the logged surfer
	 * - $fields['nick_name'] - nick name of the logged surfer
	 * - $fields['email'] - email address
	 * - $fields['editor'] - preferred on-line editor
	 * - $fields['capability'] - 'A'ssociate or 'M'ember
	 *
	 * We also remember the IP address of the authenticating workstation,
	 * and the root path of the instance that has validated the surfer.
	 *
	 * @param array session attributes
	 * @param int offset to GMT, if any
	 */
	function set($fields, $offset=0) {
		global $context;

		// save session attributes
		$_SESSION['surfer_id'] = isset($fields['id'])?$fields['id']:'';

		$_SESSION['surfer_name'] = isset($fields['nick_name'])?$fields['nick_name']:'';

		$_SESSION['surfer_email_address'] = isset($fields['email'])?$fields['email']:'';

		// provide a default capability only to recorded users
		if(!$_SESSION['surfer_id'])
			$default_capability = '';

		// closed community, accept only subscribers
		elseif(isset($context['users_with_approved_members']) && ($context['users_with_approved_members'] == 'Y'))
			$default_capability = 'S';

		// email addresses have to be validated
		elseif(isset($context['users_with_email_validation']) && ($context['users_with_email_validation'] == 'Y'))
			$default_capability = 'S';

		// open community, accept subscribers and members
		else
			$default_capability = 'M';

		$_SESSION['surfer_capability'] = isset($fields['capability'])?$fields['capability']:$default_capability;

		// editor preference
		if(isset($fields['editor']))
			$_SESSION['surfer_editor'] = $fields['editor'];
		if(!isset($_SESSION['surfer_editor']) && !$_SESSION['surfer_editor'])
			$_SESSION['surfer_editor'] = 'yacs';

		// remember surfer time zone
		if(!isset($_SESSION['surfer_gmt_offset']) || $offset)
			$_SESSION['surfer_gmt_offset'] = $offset;

		// remember the address of the authenticating workstation
		if(isset($_SERVER['REMOTE_ADDR']))
			$_SESSION['workstation_id'] = $_SERVER['REMOTE_ADDR'];

		// remember the authenticating instance
		if(isset($context['url_to_root']) && $context['url_to_root'])
			$_SESSION['server_id'] = $context['url_to_root'];

		// the surfer has been authenticated, do not challenge him anymore
		$_SESSION['surfer_is_not_a_robot'] = TRUE;

	}

	/**
	 * remember surfer data
	 *
	 * This function is used to track anonymous surfers based on what they
	 * put in web forms.
	 *
	 * Following named attributes from the provided array are copied in session storage area:
	 * - $fields['edit_name'] - nick name of the logged surfer
	 * - $fields['edit_address'] - email address
	 *
	 * @param array session attributes
	 */
	function track($fields) {
		global $context;

		// preserve permanent settings
		if(Surfer::is_logged())
			return;

		// remember one time name
		if(isset($fields['edit_name']))
			$_SESSION['surfer_name'] = $fields['edit_name'];

		// remember one time email address
		if(isset($fields['edit_address']))
			$_SESSION['surfer_email_address'] = $fields['edit_address'];

	}

	/**
	 * strip all HTML tags
	 *
	 * @link http://www.fileformat.info/info/unicode/version/1.1/index.htm Unicode 1.1
	 *
	 * @param a string or an array
	 * @param string of allowed tags, if any
	 * @return a clean string or array
	 */
	function strip_tags($input, $allowed_tags='') {

		// do it recursively
		if(is_array($input)) {
			foreach($input as $name => $value)
				$input[$name] = Surfer::strip_tags($value, $allowed_tags);
			return $input;
		}

		// unmask HTML special chars
		$input = preg_replace('/&(#0*38|amp);/i', '&', $input);
		$input = preg_replace('/%u0*26/i', '&', $input);

		$input = preg_replace('/&(#0*60|lt);/i', '<', $input);
		$input = preg_replace('/%u0*3c/i', '<', $input);

		$input = preg_replace('/&(#0*62|gt);/i', '>', $input);
		$input = preg_replace('/%u0*3e/i', '>', $input);

		$input = preg_replace('/&#0*59;/i', ';', $input);

		// strip tags
		return strip_tags($input, $allowed_tags);
	}

	/**
	 * adjust a time stamp to GMT
	 *
	 * @param string a time stamp in surfer time zone
	 * @return string a time stamp in GMT zone
	 */
	function &to_GMT($stamp) {
		global $context;

		// sanity check
		if(!isset($stamp) || ($stamp <= NULL_DATE) || (($seconds = strtotime($stamp)) == -1))
			return $stamp;

		// shift to GMT
		$stamp = strftime('%Y-%m-%d %H:%M:%S', $seconds - (Surfer::get_gmt_offset() * 3600));
		return $stamp;
	}
}

// we don't want to use url rewriting, but only cookies (exclusive with 'use.cookies' and 'use_trans_sid')
if(isset($_SERVER['REMOTE_ADDR'])) {
	Safe::ini_set('session.use_cookies', '1');
	Safe::ini_set('session.use_only_cookies', '1');
	//Safe::ini_set('session.use_trans_sid', '0'); -- don't uncomment !!!
	Safe::ini_set('url_rewriter.tags', '');
}

// retrieve session data, but not if run from the command line, and not from robot nor spider
if(isset($_SERVER['REMOTE_ADDR']) && !Surfer::is_crawler() && !headers_sent()) {

	// set or retrieve session context
	if(session_id() == '')
		session_start();

	// switch to another instance on same host
	if(isset($_SESSION['server_id']) && isset($_SESSION['url_to_root']) && strcmp($_SESSION['server_id'], $context['url_to_root']))
		Surfer::reset();

	// permanent identification has been selected
	elseif(isset($context['users_with_permanent_authentication']) && ($context['users_with_permanent_authentication'] == 'Y')) {

		// use cookie to identify user -- user id, time of login, gmt offset, salt
		if(!Surfer::is_logged() && isset($_COOKIE['screening']) && ($nouns = explode('|', $_COOKIE['screening'], 4)) && (count($nouns) == 4)) {

			// get user by id
			if(!$user = Users::get($nouns[0]))
				;

			// ensure we have the target user profile
			elseif(!isset($user['id']) || ($user['id'] != $nouns[0]))
				;

			// salted stamp has to match
			elseif(!isset($user['handle']) || strcmp($nouns[3], md5($nouns[1].'|'.$user['handle'])))
				;

			// save surfer profile in session context
			else
				Surfer::set($user, $nouns[2]);

		}

	// allow no more than one hour of inactivity
	} elseif(isset($_SESSION['watchdog']) && (time() > ($_SESSION['watchdog'] + 3600)))
		Surfer::reset();

	// refresh the watchdog
	$_SESSION['watchdog'] = time();

	// bare server-side flash detection
	if(isset($_SERVER['HTTP_ACCEPT']) && preg_match('/application\/x-shockwave-flash/i', $_SERVER['HTTP_ACCEPT']))
		$_SESSION['browser_has_flash'] = TRUE;

}

// filter HTML tags of submitted material
if(@count($_REQUEST) && (!isset($context['allow_html_input']) || ($context['allow_html_input'] != 'Y'))) {

	// from an associate
	if(Surfer::is_associate())
		;

	// from fckeditor or tinymce
	elseif(isset($_REQUEST['editor']) && (($_REQUEST['editor'] == 'fckeditor') || ($_REQUEST['editor'] == 'tinymce')))
		;

	// strip most tags, except those that have been explicitly allowed
	else {
		if(!isset($context['users_allowed_tags']))
			$context['users_allowed_tags'] = '';
		$_REQUEST = Surfer::strip_tags($_REQUEST, $context['users_allowed_tags']);
	}
}

?>